#include "headers.h"

MODULE = Arisa::Pool  PACKAGE = Arisa::Interface
PROTOTYPES: ENABLE

Arisa::Pool
pool_find(intf, name = NULL)
	Arisa::Interface intf
	char *name
CODE:
	RETVAL.minor = intf;
	if(interface_valid(intf))
		RETVAL.major = pool_find(intf, name);
	else
		RETVAL.major = NULL;
OUTPUT:
	RETVAL

void
pools(intf)
	Arisa::Interface intf
PREINIT:
	int i;
PPCODE:
	if(interface_valid(intf)) {
		LOCK(intf);
		if(intf->type == INTERFACE_SEND) {
			for(i = 0; i < intf->send.no_pools; ++i) {
				pool_t *p = intf->send.pools[i];
				LOCK(p);
				if(p->deleted == 0) {
					XPUSHs(sv_2mortal(
						create_ptr2_object_p(
							"Arisa::Pool",p,intf)));
				}
				UNLOCK(p);
			}
		}
		UNLOCK(intf);
	}

Arisa::Pool
pool_add(intf, name = NULL)
	Arisa::Interface intf
	char *name
CODE:
	RETVAL.major = NULL;
	RETVAL.minor = intf;
	if(interface_valid(intf) && name != NULL) {
		pool_t *p = alloc_pool();
		p->name = xstrdup(name);
		
		if(pool_add(intf,p) == 0)
			RETVAL.major = p;
		else
			free_pool(p);
	}
OUTPUT:
	RETVAL

	
#*******************************
MODULE = Arisa::Pool  PACKAGE = Arisa::Pool
#*******************************

int
close_send(pool, ...)
	Arisa::Pool pool
PREINIT:
	interface_t *intf;
	pool_t *p;
CODE:
	RETVAL = 0;
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	if(pool_valid(intf,p)) {
		network_t *net	= SvPTRarg("network");
		char *host	= SvPVarg("host");
		char *nick	= SvPVarg("nick");
		char *file	= SvPVarg("file");
		char *reason	= SvPVarg("reason");
		void *ptr	= (void *)SvIVarg("_ptr");
		
		RETVAL = pool_send_close(p,host,net,nick,file,ptr,reason);
	}
OUTPUT:
	RETVAL

SV*
delete(pool)
	Arisa::Pool pool
PREINIT:
	interface_t *intf;
	pool_t *p;
CODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	RETVAL = &PL_sv_no;
	if(pool_valid(intf,p)) {
		if(pool_del(intf,p) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
name(pool, new_name = NULL)
	Arisa::Pool pool
	char *new_name
PREINIT:
	interface_t *intf;
	pool_t *p;
CODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	RETVAL = &PL_sv_no;
	if(pool_valid(intf,p)) {
		if(new_name != NULL) {
			if(pool_rename(intf,p,new_name) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			LOCK(p);
			RETVAL = new_pv(p->name);
			UNLOCK(p);
		}
	}
OUTPUT:
	RETVAL

void
info(pool)
	Arisa::Pool pool
PREINIT:
	HV *hv = NULL;
	interface_t *intf;
	pool_t *p;
PPCODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	if(pool_valid(intf,p)) {
		AV *avgspeed = newAV();
		uint64_t rate; // incase we have a *really* fast pool
		int i;
		
		hv = newHV();
		
		LOCK(p);
		for(i = 0, rate = 0; i < AVGSPEED; ++i) {
			rate += p->avgspeed[i];
			av_push(avgspeed,newSVuv(p->avgspeed[i]));
		}
		hv_store(hv,"rate",4,
			newSVuv((unsigned int)(rate / (uint64_t)AVGSPEED)),0);
		hv_store(hv,"rate_data",9,newRV_noinc((SV*)avgspeed),0);
		hv_store(hv,"record_rate",11,newSVuv(p->record_rate),0);
		hv_store(hv,"record_send",11,newSVuv(p->record_send),0);
		hv_store(hv,"total_transferred",17,newSVnv(
				((double)p->stat_mib)*(1024.0*1024.0)
				+ (double)p->stat_carry),0);
		UNLOCK(p);
	}
	RET_REF(hv);

void
reset_counters(pool)
	Arisa::Pool pool
PREINIT:
	interface_t *intf;
	pool_t *p;
PPCODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	if(pool_valid(intf,p)) {
		if(pool_reset_counters(p) == 0)
			XPUSHs(&PL_sv_yes);
		else
			XPUSHs(&PL_sv_no);
	}

void
sends(pool)
	Arisa::Pool pool
PREINIT:
	interface_t *intf;
	pool_t *p;
PPCODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	if(pool_valid(intf,p)) {
		int i;
		LOCK(p);
		for(i = 0; i < p->no_csends; ++i) {
			LOCK(p->csends[i]);
			XPUSHs(sv_2mortal(newRV_noinc(
				(SV*)send_to_hv(p->csends[i]))));
			UNLOCK(p->csends[i]);
		}
		UNLOCK(p);
	}
			
void
settings(pool, ...)
	Arisa::Pool pool
PREINIT:
	HV *hv = NULL;
	interface_t *intf;
	pool_t *p;
PPCODE:
	PTR2SPLIT(pool,pool_t,p,interface_t,intf);
	if(pool_valid(intf,p)) {
		hv = newHV();
		perl_apply_settings(pool_apply_setting,p);
		perl_read_settings(pool_read_settings,p,hv);
	}
	RET_REF(hv);
